
#include <cmath>
#include <cstring>

#include "FFunction.hh"
#include "CFoldSigma.hh"
#include "TF1.h"

ClassImp(CFoldSigma)

CFoldSigma::CFoldSigma(TF1* fold) : CFold(fold)
{
  SetSigma(1);

  Double_t xmin, xmax;
  fFold->GetRange(xmin, xmax);
  fGaussian = new TF1("gaussian", Gaussian, xmin, xmax, 2);
}

CFoldSigma::~CFoldSigma()
{
  delete fGaussian;
}

void CFoldSigma::SetSigma(Double_t sigma)
{
  fSigma = sigma;
}

Double_t CFoldSigma::Evaluate(Double_t* x, Double_t* par)
{
  fFold->SetParameters(par);

  Double_t xmin, xmax;
  fFold->GetRange(xmin, xmax);
  Double_t sum = 0;
  for(Double_t xx = xmin; xx < xmax; xx += fStep){

    Double_t sigma = fSigma*sqrt(std::abs(xx));
    //    Double_t sigma = fSigma;
    fGaussian->SetParameters(xx, sigma);
    Double_t g = fGaussian->Eval(x[0]);

    Double_t f = fFold->Eval(xx);

    sum += f*g*fStep;
  }

  return sum;
}
