
#include <iostream>
#include <deque>
#include <fstream>
#include <cmath>

#include "TF1.h"

#include "CMapFunction1D2.hh"
#include "FFunction.hh"

ClassImp(CMapFunction1D2)

CMapFunction1D2::CMapFunction1D2()
{;}

CMapFunction1D2::~CMapFunction1D2()
{;}

TF1* CMapFunction1D2::GetMapFunction()
{
  return fMapFunction;
}

void CMapFunction1D2::SetMap(const std::deque<Double_t>* f,
			     const std::deque<Double_t>* x)
{
  fF = *f;
  fX = *x;
}

void CMapFunction1D2::SetFile(std::string filename)
{
  std::ifstream ifs(filename.c_str());
  Double_t f,x;
  std::deque<Double_t> df,dx;
  while(!ifs.eof()){
    ifs >> x >> f;
    df.push_back(f);
    dx.push_back(x);
  }
  SetMap(&df,&dx);

  std::deque<Double_t>::const_iterator itx = fX.begin();
  Double_t xmin=*itx;
  Double_t xmax=*itx;
  while(itx != fX.end()){
    if(*itx < xmin) xmin = *itx;
    if(*itx > xmax) xmax = *itx;
    ++itx;
  }
  fMapFunction = new TF1("mapfunction1d", this,
			 static_cast<Double_t (CMapFunction1D2::*)(Double_t*, Double_t*)>
			 (&CMapFunction1D2::MapFunction),
			 xmin, xmax, 0);
}

Double_t CMapFunction1D2::MapFunction(Double_t xx)
{
  Double_t result;

  std::deque<Double_t>::const_iterator itf = fF.begin();
  std::deque<Double_t>::const_iterator itx = fX.begin();
  Double_t lx=*itx;
  Double_t lf=*itf;
  Double_t lxtemp;
  Double_t hx=*itx;
  Double_t hf=*itf;
  Double_t hxtemp;
  while(1){
    lxtemp = xx-*itx;
    if((xx < lx) || (lxtemp >= 0 && lxtemp < xx-lx)){
      lx = *itx;
      lf = *itf;
    }

    hxtemp = *itx-xx;
    if((xx >= hx) || (hxtemp > 0 && hxtemp < hx-xx)){
      hx = *itx;
      hf = *itf;
    }

    ++itf;
    ++itx;
    if(itf == fF.end() || itx == fX.end()) break;
  }
  if(xx < lx || xx >= hx){
    std::cout << xx << " is out of range" << std::endl;
    result = 0;
  }else{
    result = (xx-lx)*(hf-lf)/(hx-lx)+lf;
  }

  return result;
}

Double_t CMapFunction1D2::MapFunction(Double_t* x, Double_t* /*par*/)
{
  Double_t xx = x[0];

  return MapFunction(xx);
}
