#ifndef ANALYZENEUTGAMMA_HH
#define ANALYZENEUTGAMMA_HH

#include "TROOT.h"

#include <string>

class TVector3;
class TCanvas;
class TDirectory;
class TFile;
class TH1D;
class TH2D;
class TH3D;

class CDataHime;
class CDataHimeSim;
class CParticle;

//#define VISUALIZE 1

class CAnalyzeNeutGamma
{
public:
  CAnalyzeNeutGamma(std::string rootfile);
  ~CAnalyzeNeutGamma();
  
  void Main(std::string prmfile, std::string detector, std::string method, Long64_t nentries=-1);

  int ReadParamaters(std::string prmfile, std::string detector);
  int CreateHist();
  int CreateExpData(std::string method, Long64_t nentries);
  int AnalysisExpData(Long64_t nentries);
  int DrawHist(Long64_t nentries);

private:
  //calculate simlation data to experiment data
  void SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method);

  //analysis method for experiment data
  void Track(CDataHime* dataexp);
  void Simple(CDataHime* dataexp);
  void Niren(CDataHime* dataexp);//not optimized

private:
  TCanvas *canvas;

  TDirectory *topdir;
  TFile *filesim, *fileexp;

  CParticle *proton;
  CParticle *neutron;

  //hist
  TH1D *hamin;
  TH1D *htheta_exp, *htheta_cm_exp;

  TH1D *htheta_lab, *hphi_lab;

  TH2D *hthetaenergy_lab, *hthetaenergy_lab_gate;
  TH2D *hthetaenergy_cm, *hthetaenergy_cm_gate;

  TH2D *hdee_lab, *hthetade_lab;
  TH3D *hthetadee_lab;

  TH1D *htheta_cm, *htheta_cm_gate;

  TH1D *hpx, *hpy, *hpz;
  TH1D *htof;

  //parameters
  Int_t thrmethod;
  Double_t thr, treso, attlength, vl;
  Int_t rotation;
  Double_t length, vlength, width, depth, gap, zdepth, zdepth_back, shiftx, shifty;
  Int_t multihit;
  Double_t aminmin, ein, sp;
  Double_t step;

  ClassDef(CAnalyzeNeutGamma, 0)
};

#endif
